# Plan
Here's a checklist of my immediate plans for the future. If you have an idea for a feature, feel free to make an issue.

## v1.3.0
### Custom colours and names
* [x] Change protocol to support this.
	* [x] Client -> Server: SERP request.
	* [x] Server -> Client: NewPlayer, PlayerColour and PlayerName events.
* [x] Render players with this colour.
* [x] Have the colour the server sets be the colour the player requested.
* [x] Use the correct colour for chat and scores.
	* [x] Chat.
	* [x] Scores.
		* [x] Move score messages from `MiscMessage` to `WorldMessage`.
		* [x] Lookup the players when rendering the scores.
* [x] Clean things up by creating `PlayerStyle` struct that wraps the colour and name.
* [x] Allow customisation of the name and colour.
	* [x] In the settings GUI with a colour picker and text input.
		* [x] Config.
		* [x] GUI.
	* [x] Use the config when joining the game.
	* [x] Allow overriding the name with command line arguments.
	* [x] Prevent what should be a clipboard emoji for copying the colour from showing up as a question mark.
	* [x] Show a live updated player with the selected colour if possible.
		* [x] Move colour picker to egui window.
		* [x] Reserve space for live preview.
		* [x] Prepare cache for player texture and program for rendering in multiple game states.
		* [x] Render player in that space.
		* [x] Fix panic when window's too small.
		* [x] Render black background under player.
		* [x] Have player look at cursor.
		* [x] Test things with DPI scaling.
		* [x] Increase player texture size.
		* [x] Fade player with window.
		* [x] Ensure that circle for selected colour doesn't get cut off (see `Ui::set_clip_rect`, might be helpful).
* [x] Fix bug that prevents the stars from being disabled.
* [x] Replace "Player N" with names in server messages when the game ends and in the lobby when players are ready.
* [x] Close colour picker window when pressing escape.
* [x] Fix bug with health bars rendering in the wrong position with DPI scaling.
* [x] Save many GUI fields to the config.
	* [x] Spectator (share this for all).
	* [x] Enter address.
	* [x] Save these changes when the GUI state changes.
* [x] Add commands to change name and colour while playing.
	* [x] Update protocol to support it.
	* [x] Get it working with hard-coded named colours like "red", "blue", etc.
	* [x] Add support for arbitrary colours as a hex code.
	* [x] Allow overriding the colour with command line arguments, using this string to colour conversion.
* [x] Fix contrast improving algorithm to preserve hue.
* [x] Text I/O changes.
	* [x] Prevent jitter when `rendered_history != height` (if a chat message consists of multiple lines).
	* [x] Add a limit to the length of the player's name.
	* [ ] ~If possible, limit the height of the text input when not open (not just the number of messages in case some messages are very long).~
		Too difficult to do.
	* [x] Mitigate impersonation of the administrator by having admin messages appear more unique and less like they came from a regular player (this is already the case but I can do better).
* [x] Fix bug where Ctrl+C doesn't close the GUI if inactive for a bit.
* [x] Render the player's name.
	* [x] In the scores.
	* [x] Ensure that the timer panel doesn't cut off the scores panel by using `Painter::text` or a similar method to render the text without a window.
	* [x] Above the players. This likely will be achieved using a method similar to `Painter::text`.
		* [x] Get a simple implementation working.
		* [x] Improve the position.
		* [x] Use the lighter colour.
		* [x] Add a transparent black background under the name.
	* [x] Truncate the names if too long.
	* [x] Prevent newlines from names resulting in things looking weird.
	* [x] Make the player name colour in text I/O consistent with the score and above player colour.
	* [ ] ~For the current player, maybe.~
		Nah, already will be rendered in the scores and indented.
* [x] Have the followed player rendered on top.

### Teams
* [x] Create `PlayerId` newtype struct wrapping a u32 to prevent accidentally improperly converting between a planned `TeamId` newtype.
* [x] Fix bug with inconsistent removal of players.
* [x] Get basic teams working.
	* [x] Give each player a `team_id: TeamId` field with a newly created `TeamId` struct.
	* [x] Add `PlayerTeam` world event to allow teams to be changed.
	* [x] Add a team owner field to the `Bullet` struct.
	* [x] Disable friendly fire.
	* [x] Add `teams: BTreeMap<TeamId, Team>` to the `World`, where `Team` is a struct that contains a display name and colour.
	* [x] Add teams to the server config.
	* [x] Replace teams with `Vec<Team>`, map not needed.
	* [x] Have players join teams when they join the world.
	* [x] Set player colour to team colour.
	* [x] On the server, remember which team the client is in when they spectate and play again.
	* [ ] ~Rename player colour to something else as it will be used by teams (or keep it, as it's still displayed when rendering the player).~
* [x] Command changes.
	* [x] Create custom team struct for config.
		* [x] Use a string for player colour.
		* [x] Add team aliases.
	* [x] Add `team` command to allow changing teams while in the lobby (and only in the lobby).
	* [x] Add `teams` command to list all teams.
	* [x] Prevent changing colour if teams are enabled.
* [x] Allow friendly fire in teams with a `WorldConfig` struct. This config will be specified in the server config.
* [x] Scoring.
	* [x] Split the scores into player scores and team scores.
	* [x] Allow configuring which scoring events go to the individual player scores or overall team scores (or both).
	* [x] Render these scores appropriately.
		If both score maps are empty, render nothing.

		If only one score map is non-empty, render those scores with the heading "Scores".

		If both score maps are non-empty, render on top "Team Scores" and below "Player Scores".
	* [x] More rendering changes.
		* [x] Change score font.
		* [x] Change render order to render names below the score panel.
	* [x] Have the initial, minimum and maximum scores grouped into a single struct and configured for the player and team.
	* [x] Get occupation to work with teams.
	* [x] Move configuration of custom spawning positions to the team struct.
	* [x] Allow unowned occupation zones.
	* [x] Renaming.
		* [x] Rename `Goal` to `Condition`, `Condition` to `Comparison` and `Comparison` to `Op`.
		* [x] Rename `goal` to `win` in the server config.
	* [x] Allow win and elimination conditions to be based on players or teams.
		* [x] Prepare generic code.
		* [x] Update win message to include team names when needed.
		* [x] Get team survival working.
			* [x] Add option for removing teams with no players from the scores. Then the `LastWith` condition can be used.
			* [x] Allow hiding of scores to the client (to be used if they are useless).
		* [x] Update comments in `condition.rs` to remove references to "goals" and have them more generic.
	* [x] When spectating, indent the score of the followed player, and team of the followed player (otherwise if not following anyone, indent nobody's player score but the current team's team score).
		This will involve remembering the last team id the current player was in.
	* [x] Fix bullet bug when there are no teams.
	* [x] Fix score rendering bug.
	* [x] Allow different ordering on player and team scores.
* [x] Render the ammo status above the player names.
* [x] Fix bug with non-instantaneous camera movement when spectating and following a player to then suddenly start playing.
* [x] Team chats.
	This will allow local communication with just the team. By default, 'T' will be for team chats while 'G' will be for global chats (sent to all players). If teams aren't enabled, team chats would be sent to all players.
	* [x] Add it to the protocol.
	* [x] Allow sending different types of chats on the client.
	* [x] Route team chats to some players.
	* [x] Update how chatting is displayed on the client to prefix team messages with "(Team) " and use the prompt "Global > " for global chats.
		* [x] Think about colouring these new parts.
	* [x] Possibly use a single text buffer shared by chats (team and global) and commands to easily allow switching while writing in case a mistake was made (or maybe just between chats).
	* [x] Ensure that new players cannot join into spectator mode, change teams and snoop on team-only communications.
	* [x] Ensure backward compatibility with v1.2.1 in the config.
* [x] Resolve all TODOs.

### Occupation improvements
One problem with the `occupation.ron` configuration when playing 1v1 is that if two players are racing for the other player's base at the same time, the player with the higher score at first gains a significant advantage. This is because for that player to have a chance of winning the game, they need to stop the other player, and they do that by leaving the other player's base and moving back towards their own base. This extreme amount of time would give that other player a huge point boost.

The need to avoid this can make the game not as fun as it should be.

I might resolve this by adding a "team-only portal" that only accepts one team id at the opposite base to allow teleportation back to your own base.

* [x] Move custom spawn points to team definition.
* [x] Improve occupation game mode.
	* [x] Create team-only portal.
	* [x] Use it in the occupation game.

### Finishing touches
* [x] Check that all server configs work (with `gen_configs.sh`).
* [x] Improve team configs.
* [x] Update documentation.
	* [x] Changing name and colour.
	* [x] Teams.
		* [x] For the user.
		* [x] Team chat.
		* [x] For the server config.
	* [x] Ensure that all configs are up-to-date.
* [x] Check dead code by replacing `pub` with the uglier `pub(crate)` temporarily and using clippy.
* [x] Update SERP version.
* [x] Update appstream metainfo with new release.
